import React from 'react'

import Waiting from './components/Waiting'
import Header from './components/Header'

import FightCategory from './components/FightCategory'
import FightDisplay from './components/FightDisplay'
import FightNext from './components/FightNext'

import FightOverview from './components/FightOverview'

import Form from './components/Form'
import Start from './components/Start'

import Constants from './util/Constants'
import Display from './util/Display'
import FightsHelper from './util/FightsHelper'
import FormHelper from './util/FormHelper'
import StartHelper from './util/StartHelper'

class App extends React.Component {

  constructor() {
    super()
    this.state = {
      display: Display.WAITING
    }
    this.messageCounter = 0
    this.sseSource = new EventSource(Constants.SERVER_SSE)
  }

  componentDidMount() {
    this.sseSource.onerror = e => {
      console.log('error', e)
      this.setState({
        display: Display.WAITING,
        title: '',
        fights: {},
        form: {},
        start: {},
        next: {}
      })
    }
    
    this.sseSource.addEventListener('keepalive', e => {
      this.checkMessageCounter(e)
    })

    this.sseSource.addEventListener('resetdisplay', e => {
      this.checkMessageCounter(e)
      this.resetDisplay()
    })
    
    this.sseSource.addEventListener('form', e => {
      this.checkMessageCounter(e)
      const form = new FormHelper(JSON.parse(e.data).form).sortByPlacement()
      this.setState({
        display: Display.FORM,
        title: JSON.parse(e.data).competition.name,
        form,
        start: JSON.parse(e.data).start
      })
    })
    this.sseSource.addEventListener('start', e => {
      this.checkMessageCounter(e)
      const form = new FormHelper(JSON.parse(e.data).form).sortByPlacement()
      const id = JSON.parse(e.data).id
      const start = form.starts.find(start => start.id === id)
      const next = form.starts.filter(start => new StartHelper(start).hasResult() === false).filter(start => start.id !== id)[0]
      this.setState({
        display: Display.START,
        title: JSON.parse(e.data).competition.name,
        form,
        start,
        next,
        result: false
      })
    })
    this.sseSource.addEventListener('result', e => {
      this.checkMessageCounter(e)
      const form = new FormHelper(JSON.parse(e.data).form).sortByPlacement()
      const id = JSON.parse(e.data).id
      const start = form.starts.find(start => start.id === id)
      const next = form.starts.filter(start => new StartHelper(start).hasResult() === false).filter(start => start.id !== id)[0]
      this.setState({
        display: Display.START,
        title: JSON.parse(e.data).competition.name,
        form,
        start,
        next,
        result: true
      })
    })

    this.sseSource.addEventListener('fights', e => {
      this.checkMessageCounter(e)
      this.setState({
        display: Display.FIGHTS,
        title: JSON.parse(e.data).competition.name,
        fights: JSON.parse(e.data).fights
      })
    })
    this.sseSource.addEventListener('fight', e => {
      this.checkMessageCounter(e)
      const data = JSON.parse(e.data)

      /* 0000338: Gewinner / Verlierer werden nicht immer aktualisiert */
      const fightsHelper = new FightsHelper(data.fights)
      data.fights.forEach(fight => {
        fight.red = fightsHelper.getRed(fight.id)
        fight.redCountry = fightsHelper.getRedCountry(fight.id)
        fight.blue = fightsHelper.getBlue(fight.id)
        fight.blueCountry = fightsHelper.getBlueCountry(fight.id)
      })

      this.setState({
        display: Display.FIGHT,
        title: data.competition.name,
        fights: data.fights,
        id: data.id
      })
    })
  }

  componentWillUnmount() {
    document.body.className = null;
    this.sseSource.close()
  }

  componentDidCatch(error, info) {
    fetch(Constants.SERVER_LOGGER, {
      method: 'POST',
      mode: 'cors',
      cache: 'no-cache',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify({ 
        level: 'error', 
        message: `${error.message}\n${error.stack}`
      })
    }).then(
      this.resetDisplay()
    )
  }

  resetDisplay() {
    // 0000356: Restarting clients
    document.location.reload()    
  }

  checkMessageCounter(e) {
    const counter = parseInt(e.lastEventId)
    if (this.messageCounter + 1 !== counter) {
      // tdb: Fehler erkannt, was ist nun zu tun?
    }
    this.messageCounter = counter
  } 

  setBackground() {
    switch (this.state.display) {
      case Display.WAITING:
        document.body.className = 'waiting-for-data'
        break
      case Display.FIGHTS:
        document.body.className = 'fight-overview-background'
        break
      case Display.FIGHT:
        document.body.className = 'fight-background'
        break
      case Display.FORM:
        document.body.className = 'form-background'
        break
      case Display.START:
        document.body.className = 'start-background'
        break
      default:
        document.body.className = 'waiting-for-data'
        break
    }
  }

  render() {
    this.setBackground()

    switch (this.state.display) {
      case Display.WAITING:
        return (
          <React.Fragment>
            <Waiting />
          </React.Fragment>
        )
      case Display.FIGHT:
        /* 0000337: Show data of next fight */
        const index = this.state.fights.findIndex(fight => fight.id === this.state.id)
        const fightDisplay = this.state.fights[index]
        const fightNext = this.state.fights[index + 1 ]
        return (
          <React.Fragment>
            <Header title={this.state.title}/>
            <FightCategory data={fightDisplay}/>
            <FightDisplay fight={fightDisplay}/>
            <FightNext data={fightNext}/>
          </React.Fragment>
        )
      case Display.FIGHTS:
        return (
          <React.Fragment>
            <Header title={this.state.title}/>
            <FightOverview fights={this.state.fights}/>
          </React.Fragment>
        )
      case Display.FORM:
        return (
          <React.Fragment>
            <Header title={this.state.title}/>
            <Form form={this.state.form} start={this.state.start}/>
          </React.Fragment>
        )
      case Display.START:
        return (
          <React.Fragment>
            <Header title={this.state.title}/>
            <Start form={this.state.form} start={this.state.start} result={this.state.result} next={this.state.next}/>
          </React.Fragment>
        )
      default:
        return (
          <React.Fragment>
            <Waiting />
          </React.Fragment>
        )
    }
  }
}

export default App;
