const RoundsHelper = require('./RoundsHelper')
const Winner = require('./Winner')
const Constants = require('./Constants')

class FightsHelper {
    constructor(fights) {
        this.fights = fights
    }

    /**
     * Returns whether the given fight is the current one.
     * @param {String} id id of the Fight
     */
    isCurrent(id) {
        const finishedRounds = this.fights.filter(fight => new RoundsHelper(fight.rounds).isFinished() === false)
        return finishedRounds.length > 0 && finishedRounds[0].id === id
    }

    /**
     * Returns the Name of the Red fighter of the given round.
     * @param {String} id id of the Fight
     */
    getRed(id) {
        // invalid fight number
        if (this.fights === undefined || id === '') {
            return Constants.UNKNOWN_FIGHTER
        }

        const fight = this.fights.find(fight => fight.id === id)
        if (fight === undefined) {
            return Constants.UNKNOWN_FIGHTER
        }

        const red = fight.red
        // check for + or -
        if (red.startsWith('+')) {
            const refFightId = red.substring(1)
            return this.getWinner(refFightId)
        } else if (red.startsWith('-')) {
            const refFightId = red.substring(1)
            return this.getLoser(refFightId)
        } else {
            return red
        }
    }

    /**
     * Returns the Country of the Red fighter of the given round.
     * @param {String} id id of the Fight
     */
    getRedCountry(id) {
        // invalid fight number
        if (this.fights === undefined || id === '') {
            return Constants.UNKNOWN_COUNTRY
        }

        const fight = this.fights.find(fight => fight.id === id)
        if (fight === undefined) {
            return Constants.UNKNOWN_COUNTRY
        }

        const red = fight.red
        // check for + or -
        if (red.startsWith('+')) {
            const refFightId = red.substring(1)
            return this.getWinnerCountry(refFightId)
        } else if (red.startsWith('-')) {
            const refFightId = red.substring(1)
            return this.getLoserCountry(refFightId)
        } else {
            return fight.redCountry
        }
    }

    /**
     * Returns the Name of the Blue fighter of the given round.
     * @param {String} id id of the Fight
     */
    getBlue(id) {
        // invalid fight number
        if (this.fights === undefined || id === '') {
            return Constants.UNKNOWN_FIGHTER
        }

        const fight = this.fights.find(fight => fight.id === id)
        if (fight === undefined) {
            return Constants.UNKNOWN_FIGHTER
        }

        const blue = fight.blue
        // check for + or -
        if (blue.startsWith('+')) {
            const refFightId = blue.substring(1)
            return this.getWinner(refFightId)
        } else if (blue.startsWith('-')) {
            const refFightId = blue.substring(1)
            return this.getLoser(refFightId)
        } else {
            return blue
        }
    }

    /**
     * Returns the Country of the Blue fighter of the given round.
     * @param {String} id id of the Fight
     */
    getBlueCountry(id) {
        // invalid fight number
        if (this.fights === undefined || id === '') {
            return Constants.UNKNOWN_COUNTRY
        }

        const fight = this.fights.find(fight => fight.id === id)
        if (fight === undefined) {
            return Constants.UNKNOWN_COUNTRY
        }

        const blue = fight.blue
        // check for + or -
        if (blue.startsWith('+')) {
            const refFightId = blue.substring(1)
            return this.getWinnerCountry(refFightId)
        } else if (blue.startsWith('-')) {
            const refFightId = blue.substring(1)
            return this.getLoserCountry(refFightId)
        } else {
            return fight.blueCountry
        }
    }

    /**
     * Returns the Name of the winning fighter of the given round.
     * @param {String} id id of the Fight
     */
    getWinner(id) {
        const winner = this.private_getWinner(id)
        if (winner === Winner.RED) {
            return this.getRed(id)
        } else if (winner === Winner.BLUE) {
            return this.getBlue(id)
        } else {
            const fight = this.fights.find(fight => fight.id === id)
            return Constants.WINNER_OF_FIGHT + ' ' + fight.number
        }
    }

    /**
     * Returns the Country of the winning fighter of the given round.
     * @param {String} id id of the Fight
     */
    getWinnerCountry(id) {
        const winner = this.private_getWinner(id)
        if (winner === Winner.RED) {
            return this.getRedCountry(id)
        } else if (winner === Winner.BLUE) {
            return this.getBlueCountry(id)
        } else {
            return Constants.UNKNOWN_COUNTRY
        }
    }

    /**
     * Returns the Name of the losing fighter of the given round.
     * @param {String} id id of the Fight
     */
    getLoser(id) {
        const winner = this.private_getWinner(id)
        if (winner === Winner.RED) {
            return this.getBlue(id)
        } else if (winner === Winner.BLUE) {
            return this.getRed(id)
        } else {
            const fight = this.fights.find(fight => fight.id === id)
            return Constants.LOSER_OF_FIGHT + ' ' + fight.number
        }
    }

    /**
     * Returns the Country of the losing fighter of the given round.
     * @param {String} id id of the Fight
     */
    getLoserCountry(id) {
        const winner = this.private_getWinner(id)
        if (winner === Winner.RED) {
            return this.getBlueCountry(id)
        } else if (winner === Winner.BLUE) {
            return this.getRedCountry(id)
        } else {
            return Constants.UNKNOWN_COUNTRY
        }
    }

    private_getWinner(id) {
        const fight = this.fights.find(fight => fight.id === id)
        const roundsHelper = new RoundsHelper(fight.rounds)
        return roundsHelper.getWinner()
    }
}

module.exports = FightsHelper