const RoundHelper = require('./RoundHelper')
const Winner = require('./Winner')

class RoundsHelper {
    constructor(rounds) {
        this.rounds = rounds
    }

    /**
     * Returns the number of the current round. 
     * -1 if there is no current round, because the fight is finished.
     * If the fight is tied, the number of the current round is returned; meaninng, we are still in that round and cannot move on.
     */
    getNumberCurrentRound() {
        const results = this.rounds.results.filter(round => new RoundHelper(round).hasOnlyWarnings() === false)

        // wenn es noch gar keine Runden gibt, dann ist die erste Runde die aktuelle
        if (results.length === 0) {
            return 1
        }

        const winner = this.getWinner()
        if (winner === Winner.BLUE || winner === Winner.RED) {
            // Wenn der Kampf beendet ist, dann gibt es keine aktuelle Runde
            return -1
        } else if (winner === Winner.TIE) {
            // Bei Gleichstand die aktuelle Runde wiederholen
            return results.length
        } else {
            // Es gibt noch kein Ergebnis -> die Runde nach der letzten mit Punkten ist die aktuelle Runde
            return results.length + 1
        }
    }

    /**
     * Ermittelt den Gewinner des Kampfes. 
     * Ist Winner.NONE, wenn der Kampf noch andauert.
     * Ist Winner.TIE, wenn der Kampf unentschieden ist.
     */
    getWinner() {
        const results = this.rounds.results.filter(round => new RoundHelper(round).hasOnlyWarnings() === false)

        // Wenn ein Status enthalten ist -> Gewinner steht fest
        const status = results.find(round => new RoundHelper(round).hasStatus())
        if (status !== undefined) {
            return new RoundHelper(status).getWinner()
        }

        // Prüfen, ob alle Runden ein Ergebnis haben
        const winners = results.map(round => new RoundHelper(round).getWinner())
        if (winners.length === this.rounds.max) {
            const redWins = winners.reduce((wins, winner) => winner === Winner.RED ? wins + 1 : wins, 0)
            const blueWins = winners.reduce((wins, winner) => winner === Winner.BLUE ? wins + 1 : wins, 0)
            if (redWins > blueWins) {
                return Winner.RED
            } else if (blueWins > redWins) {
                return Winner.BLUE
            } else {
                return Winner.TIE
            }
        }

        // nicht alle Runden haben ein Ergebnis
        return Winner.NONE
    }

    /**
     * Returns true if the whole fight is decided, ie is not TIE nor NONE.
     * Returns false otherwise.
     */
    isFinished() {
        const winner = this.getWinner()
        return winner !== Winner.TIE && winner !== Winner.NONE
    }

    /**
     * Liefert true, wenn der Winner unentschieden ist (Kampf muss maximale Anzahl Runden erreicht haben)
     */
    isTied() {
        return this.getWinner() === Winner.TIE
    }

    /**
     * Ermittelt die Anzahl der noch zu kämpfenden Runden
     */
    getNumberRoundsAhead() {
        return this.getNumberCurrentRound() !== -1 ? this.rounds.max - this.getNumberCurrentRound() : 0
    }

    /**
     * Gibt die letzte Runde aus. Wenn der Kampf noch nicht beendet ist, dann wird undefined zurückgegeben.
     */
    getLastRound() {
        if (this.isFinished()) {
            return this.rounds.results[this.rounds.results.length - 1]
        }
    }
}

module.exports = RoundsHelper