const express = require('express')
const dataProvider = require('./../util/dataProvider')
const logger = require('./../util/logger')

const Point = require('./../display/src/util/Point')
const StopWatchState = require('./../display/src/util/StopWatchState')
const RoundsHelper = require('./../display/src/util/RoundsHelper')
const Constants = require('./../display/src/util/Constants')

module.exports = (displayEmitter) => {
    const router = express.Router()

    /**
     * Zeigt Übersicht aller Kämpfe an
     */
    router.get('/', (req, res) => {
        logger.info(logger.LABELS.fights, '/')
        res.render('fights', {
            title: `${dataProvider.getCompetition('name')}: Lei Tai`,
            fights: dataProvider.getFights(),
            apiDisplayChecked
        })
    })
    
    /**
     * Springt von Übersicht aller Kämpfe zu einem Kampf
     */
    router.post('/', (req, res) => {
        const id = req.body.displayId
        if (id) {
            res.redirect(`/fights/${id}`)
        } else {
            if (apiDisplayChecked) {
                displayEmitter.send('fights', dataProvider.getCompetition(), dataProvider.getFights())
            }
            res.redirect('back');
        }
    })

    /**
     * Zeigt einen einzelnen Kampf an
     */
    router.get('/:id', (req, res) => {
        logger.info(logger.LABELS.fights, `/:${req.params.id}`)
        const id = req.params.id
        const fight = dataProvider.getFight(id)

        const roundsHelper = new RoundsHelper(fight.rounds)
        const numberRoundsAhead = roundsHelper.getNumberRoundsAhead()
        const numberCurrentRound = roundsHelper.getNumberCurrentRound()
        const displayTiedMessage = roundsHelper.isTied()
        const displayCurrentRound = roundsHelper.getNumberCurrentRound() > 0 && roundsHelper.isTied() === false
        const displayFinishedMessage = roundsHelper.isFinished()

        res.render('fight', {
            title: `${dataProvider.getCompetition('name')}: Lei Tai: Kampf ${fight.number}: ${fight.category}`,
            fights: dataProvider.getFights(),
            fight,
            Point,
            Constants,
            apiDisplayChecked,
            numberCurrentRound,
            numberRoundsAhead,
            displayFinishedMessage,
            displayTiedMessage,
            displayCurrentRound
        })
    })

    /**
     * Aktualiserung Kampf
     */
    router.post('/:id', (req, res)  => { 
        logger.info(logger.LABELS.fights, `/:${req.params.id}: ${JSON.stringify(req.body)}`)
        
        const fight = dataProvider.getFight(req.params.id)

        if (req.body.type === 'stopWatch') {
            // stopwatch
            fight.rounds.stopWatch.state = StopWatchState.get(req.body.stopWatchState)
            fight.rounds.stopWatch.time = req.body.stopWatchTime
        } else if (req.body.type === 'warning') {
            // warning
            fight.rounds.warnings.red = Point.get(`point-${req.body.redWarningValue}`)
            fight.rounds.warnings.blue = Point.get(`point-${req.body.blueWarningValue}`)
        } else {
            const results = []
            
            // points - map each round as it comes in into the object
            Object.keys(req.body).map(key => {
                if (key.startsWith('roundValue')) {
                    const points = req.body[key]
                    if (points) {
                        const numberRedPoints = (points.match(new RegExp("r", "g")) || []).length
                        const numberBluePoints = (points.match(new RegExp("b", "g")) || []).length
                        const numberTiePoints = (points.match(new RegExp("x", "g")) || []).length
                        const redPoints = Point.get(`point-${numberRedPoints}`)
                        const bluePoints = Point.get(`point-${numberBluePoints}`)
                        const tiePoints = Point.get(`point-${numberTiePoints}`)
                        results.push({
                            red: redPoints,
                            blue: bluePoints,
                            tie: tiePoints
                        })
                    }
                }
            })

            // handle status if present -> overrites results for the current round
            const redStatus = req.body.redStatus
            const blueStatus = req.body.blueStatus
            if (redStatus && redStatus.length > 0) {
                if (results.length === new RoundsHelper(fight.rounds).getNumberCurrentRound()) {
                    // es wurde etwas in die aktuelle Runde eingetragen -> wird von status überschrieben
                    results.splice(-1)
                }
                results.push({
                    red: Point.get(redStatus),
                    blue: Point.WINNER,
                })
            } else if (blueStatus && blueStatus.length > 0) {
                if (results.length === new RoundsHelper(fight.rounds).getNumberCurrentRound()) {
                    // es wurde etwas in die aktuelle Runde eingetragen -> wird von status überschrieben
                    results.splice(-1)
                }
                results.push({
                    red: Point.WINNER,
                    blue: Point.get(blueStatus)
                })
            }

            // replace results in fight
            fight.rounds.results = results

            // warnings
            fight.rounds.warnings.red = Point.get(`point-${req.body.redWarningValue}`)
            fight.rounds.warnings.blue = Point.get(`point-${req.body.blueWarningValue}`)

            // number of rounds
            fight.rounds.max = parseInt(req.body.numberOfRounds)

            // always reset stopwatch
            fight.rounds.stopWatch.time = fight.rounds.stopWatch.start
            fight.rounds.stopWatch.state = StopWatchState.PAUSE
        }

        // emit fight to display -> update stats or show animation for winner
        if (apiDisplayChecked) {
            displayEmitter.send('fight', dataProvider.getCompetition(), dataProvider.getFights(), fight)
        }
        
        dataProvider.saveFights()

        // Roundshelper mit aktuellen Rundendaten erzeugen
        const roundsHelper = new RoundsHelper(fight.rounds)

        const numberRoundsAhead = roundsHelper.getNumberRoundsAhead()
        const numberCurrentRound = roundsHelper.getNumberCurrentRound()
        const displayTiedMessage = roundsHelper.isTied()
        const displayCurrentRound = roundsHelper.getNumberCurrentRound() > 0 && roundsHelper.isTied() === false
        const displayFinishedMessage = roundsHelper.isFinished()

        res.render('fight', {
            title: `${dataProvider.getCompetition('name')}: Lei Tai: Kampf ${fight.number}: ${fight.category}`,
            fights: dataProvider.getFights(),
            fight,
            Point,
            Constants,
            apiDisplayChecked,
            numberCurrentRound,
            numberRoundsAhead,
            displayFinishedMessage,
            displayTiedMessage,
            displayCurrentRound
        })
    })

    return router
}